@@ -2,7 +2,7 @@
  Project: Graphos
  GRBezierPath.m
 
- Copyright (C) 2000-2015 GNUstep Application Project
+ Copyright (C) 2000-2018 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -54,7 +54,7 @@ static double k = 0.025;
         inView:(GRDocView *)aView
         zoomFactor:(CGFloat)zf
 {
-  self = [super init];
+  self = [super initFromData:description inView:aView zoomFactor:zf];
   if(self != nil)
     {
       NSArray *psops, *linearr;
@@ -64,20 +64,11 @@ static double k = 0.025;
       double distx, disty;
       NSUInteger i, count;
       NSArray *points;
-      CGFloat strokeCol[4];
-      CGFloat fillCol[4];
-      CGFloat strokeAlpha;
-      CGFloat fillAlpha;
-      id obj;
       BOOL symm;
       
       psops = nil;
       linearr = nil;
-      docView = aView;
-      zmFactor = zf;
-      editor = [self allocEditor];
-      myPath = [[NSBezierPath bezierPath] retain];
-      [myPath setCachesBezierPath: NO];
+
       controlPoints = [[NSMutableArray alloc] initWithCapacity: 1];
       points = [description objectForKey: @"points"];
       for (i = 0; i < [points count]; i++)
@@ -104,7 +95,7 @@ static double k = 0.025;
           [cp release];
         }
       [self confirmNewCurve];
-      [self remakePath];
+ 
       psops = [description objectForKey: @"psdata"];
       for(i = 0; i < [psops count]; i++)
         {
@@ -156,81 +147,7 @@ static double k = 0.025;
             }
         }
       
-      flatness = [[description objectForKey: @"flatness"] floatValue];
-      linejoin = [[description objectForKey: @"linejoin"] intValue];
-      linecap = [[description objectForKey: @"linecap"] intValue];
-      miterlimit = [[description objectForKey: @"miterlimit"] floatValue];
-      linewidth = [[description objectForKey: @"linewidth"] floatValue];
-      obj = [description objectForKey: @"stroked"];
-      if ([obj isKindOfClass:[NSString class]])
-	obj = [NSNumber numberWithInt:[obj intValue]];
-      stroked = [obj boolValue];
-      strokeAlpha = [[description objectForKey: @"strokealpha"] floatValue];
-      str = [description objectForKey: @"strokecolor"];
-      linearr = [str componentsSeparatedByString: @" "];
-      if ([linearr count] == 3)
-	{
-	  strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
-	  strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
-	  strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
-	  strokeColor = [NSColor colorWithCalibratedRed: strokeCol[0]
-						  green: strokeCol[1]
-						   blue: strokeCol[2]
-						  alpha: strokeAlpha];
-	  [strokeColor retain];
-	}
-      else
-	{
-	  strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
-	  strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
-	  strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
-	  strokeCol[3] = [[linearr objectAtIndex: 3] floatValue];
-	  strokeColor = [NSColor colorWithDeviceCyan: strokeCol[0]
-					     magenta: strokeCol[1]
-					      yellow: strokeCol[2]
-					       black: strokeCol[3]
-					       alpha: strokeAlpha];
-	  strokeColor = [[strokeColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
-	  }
-      obj = [description objectForKey: @"filled"];
-      if ([obj isKindOfClass:[NSString class]])
-	obj = [NSNumber numberWithInt:[obj intValue]];
-      filled = [obj boolValue];
-      fillAlpha = [[description objectForKey: @"fillalpha"] floatValue];
-      str = [description objectForKey: @"fillcolor"];
-      linearr = [str componentsSeparatedByString: @" "];
-      if ([linearr count] == 3)
-	{
-	  fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
-	  fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
-	  fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
-	  fillColor = [NSColor colorWithCalibratedRed: fillCol[0]
-						green: fillCol[1]
-						 blue: fillCol[2]
-						alpha: fillAlpha];
-	  [fillColor retain];
-	}
-      else
-	{
-	  fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
-	  fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
-	  fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
-	  fillCol[3] = [[linearr objectAtIndex: 3] floatValue];
-	  fillColor = [NSColor colorWithDeviceCyan: fillCol[0]
-					   magenta: fillCol[1]
-					    yellow: fillCol[2]
-					     black: fillCol[3]
-					     alpha: fillAlpha];
-	  fillColor = [[fillColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
-	}
-      obj = [description objectForKey: @"visible"];
-      if ([obj isKindOfClass:[NSString class]])
-	obj = [NSNumber numberWithInt:[obj intValue]];
-      visible = [obj boolValue];
-      obj = [description objectForKey: @"locked"];
-      if ([obj isKindOfClass:[NSString class]])
-	obj = [NSNumber numberWithInt:[obj intValue]];
-      locked = [obj boolValue];
+
     }
 
   return self;
@@ -263,9 +180,9 @@ static double k = 0.025;
 
   str = [NSString stringWithFormat: @"%.3f", (float)flatness];
   [dict setObject: str forKey: @"flatness"];
-  str = [NSString stringWithFormat: @"%i", linejoin];
+  str = [NSString stringWithFormat: @"%u", (unsigned)linejoin];
   [dict setObject: str forKey: @"linejoin"];
-  str = [NSString stringWithFormat: @"%i", linecap];
+  str = [NSString stringWithFormat: @"%u", (unsigned)linecap];
   [dict setObject: str forKey: @"linecap"];
   str = [NSString stringWithFormat: @"%.3f", (float)miterlimit];
   [dict setObject: str forKey: @"miterlimit"];
@@ -322,6 +239,7 @@ static double k = 0.025;
       GRBezierControlPoint *cpCopy;
 
       cpCopy = [cp copy];
+      [cpCopy setPath:objCopy];
       [cpsCopy addObject: cpCopy];
       [cpCopy release];
     }
@@ -355,7 +273,7 @@ static double k = 0.025;
     [cp release];
 
     if([controlPoints count] == 1)
-      [myPath moveToPoint: GRpointZoom(aPoint, zmFactor)];
+      [displayPath moveToPoint: GRpointZoom(aPoint, zmFactor)];
 }
 
 - (void)addLineToPoint:(NSPoint)aPoint
@@ -370,7 +288,7 @@ static double k = 0.025;
   if([prevpoint isActiveHandle])
     {
       handle = [prevpoint bzHandle];
-      [myPath curveToPoint: [(GRBezierControlPoint *)currentPoint center]
+      [displayPath curveToPoint: [(GRBezierControlPoint *)currentPoint center]
               controlPoint1: handle.firstHandle
               controlPoint2: [(GRBezierControlPoint *)currentPoint center]];
       [self confirmNewCurve];
@@ -380,12 +298,12 @@ static double k = 0.025;
   if([self isPoint: (GRBezierControlPoint *)currentPoint onPoint: mtopoint])
     {
       [currentPoint moveToPoint: [mtopoint center]];
-      [myPath lineToPoint: GRpointZoom([mtopoint center], zmFactor)];
+      [displayPath lineToPoint: GRpointZoom([mtopoint center], zmFactor)];
       [(GRBezierPathEditor *)editor setIsDone:YES];
     }
   else
     {
-      [myPath lineToPoint: GRpointZoom(aPoint, zmFactor)];
+      [displayPath lineToPoint: GRpointZoom(aPoint, zmFactor)];
     }
 }
 
@@ -406,13 +324,13 @@ static double k = 0.025;
       else
         {
           [mtopoint calculateBezierHandles: handlePos];
-          type = [myPath elementAtIndex: 1];
+          type = [displayPath elementAtIndex: 1];
           if(type == NSCurveToBezierPathElement)
             {
-              [myPath elementAtIndex: 1 associatedPoints: pts];
+              [displayPath elementAtIndex: 1 associatedPoints: pts];
               pts[0] = GRpointZoom([mtopoint bzHandle].firstHandle, zmFactor);
               
-              [myPath setAssociatedPoints: pts atIndex: 1];           
+              [displayPath setAssociatedPoints: pts atIndex: 1];           
             }
           else 
             {
@@ -433,11 +351,11 @@ static double k = 0.025;
         pts[0] = GRpointZoom(handle1.firstHandle, zmFactor);
         pts[1] = GRpointZoom(handle2.secondHandle, zmFactor);
         pts[2] = GRpointZoom([(GRBezierControlPoint *)currentPoint center], zmFactor);
-        [myPath setAssociatedPoints: pts atIndex: [controlPoints count] -1];
+        [displayPath setAssociatedPoints: pts atIndex: [controlPoints count] -1];
       }
     else
       {
-        [myPath curveToPoint: GRpointZoom([(GRBezierControlPoint *)currentPoint center], zmFactor)
+        [displayPath curveToPoint: GRpointZoom([(GRBezierControlPoint *)currentPoint center], zmFactor)
 		controlPoint1: GRpointZoom(handle1.firstHandle, zmFactor)
 		controlPoint2: GRpointZoom(handle2.secondHandle, zmFactor)];
         calculatingHandles = YES;
@@ -450,11 +368,10 @@ static double k = 0.025;
     GRBezierHandle handle1, handle2;
     hitData hitdata;
     NSPoint pp[81], newpp[7];
-    NSUInteger i, pcount, index;
+    int i;
+    NSUInteger pcount, index;
     double y, s, ax, ay;
 
-    return;
-    // #### FIXME what the crap is this? we always return... the rest of the method is useless!!!
     pcount = 0;
     y = (int)p.y -4;
     while(pcount < 81) {
@@ -480,7 +397,9 @@ static double k = 0.025;
     index = [self indexOfPoint: cp];
     if (index == NSNotFound)
       return;
-
+    if (index > 0)
+      index--;
+    
     ncp = [[GRBezierControlPoint alloc] initAtPoint: hitdata.p
                                           forPath: self zoomFactor: zmFactor];
     [controlPoints insertObject: ncp atIndex: index];
@@ -522,7 +441,7 @@ static double k = 0.025;
     newpp[3].y = s * newpp[2].y + hitdata.t * newpp[4].y;
 
 
-    printf("%i %i - %i %i\n", (int)[(GRBezierControlPoint *)currentPoint center].x,
+    NSLog(@"%i %i - %i %i", (int)[(GRBezierControlPoint *)currentPoint center].x,
            (int)[(GRBezierControlPoint *)currentPoint center].y, (int)newpp[3].x, (int)newpp[3].y);
 
 
@@ -533,9 +452,34 @@ static double k = 0.025;
     [self remakePath];
 }
 
+- (void)deletePoint:(GRBezierControlPoint *)p
+{
+  NSUInteger i;
+  GRBezierControlPoint *cpToDelete;
+
+  if ([controlPoints count] < 2)
+    return;
+
+  i = 0;
+  cpToDelete = NULL;
+  while (i < [controlPoints count] && cpToDelete == NULL)
+    {
+      GRBezierControlPoint *cp;
+      
+      cp = [controlPoints objectAtIndex:i];
+      if (cp == p)
+        cpToDelete = cp;
+      else
+        i++;
+    }
+
+  [controlPoints removeObjectAtIndex:i];
+  [self remakePath];
+}
+
 - (BOOL)isPoint:(GRBezierControlPoint *)cp1 onPoint:(GRBezierControlPoint *)cp2
 {
-    return pointInRect([cp2 centerRect], [cp1 center]);
+  return pointInRect([cp2 centerRect], [cp1 center]);
 }
 
 - (GRBezierControlPoint *)pointOnPoint:(GRBezierControlPoint *)aPoint
@@ -563,7 +507,9 @@ static double k = 0.025;
   calculatingHandles = NO;
   if([controlPoints count] == 1)
     return;
-  if([self isPoint: (GRBezierControlPoint *)currentPoint onPoint: [controlPoints objectAtIndex: 0]])
+  if (currentPoint == nil)
+    [(GRBezierPathEditor *)editor setIsDone:YES];
+  else if([self isPoint: (GRBezierControlPoint *)currentPoint onPoint: [controlPoints objectAtIndex: 0]])
     [(GRBezierPathEditor *)editor setIsDone:YES];
   
   [self remakePath];
@@ -572,14 +518,16 @@ static double k = 0.025;
 - (void)remakePath
 {
   GRBezierControlPoint *cp, *prevcp, *mtopoint;
-  NSInteger i;
+  NSUInteger i;
 
-  [myPath removeAllPoints];
+  [path removeAllPoints];
+  [displayPath removeAllPoints];
   if (!controlPoints || [controlPoints count] == 0)
     return;
 
   mtopoint = [controlPoints objectAtIndex: 0];
-  [myPath moveToPoint: GRpointZoom([mtopoint center], zmFactor)];
+  [path moveToPoint: [mtopoint center]];
+  [displayPath moveToPoint: GRpointZoom([mtopoint center], zmFactor)];
   for(i = 1; i < [controlPoints count]; i++)
     {
       GRBezierHandle handle1, handle2;
@@ -598,11 +546,15 @@ static double k = 0.025;
       
       if (isLine)
         {
-          [myPath lineToPoint: GRpointZoom([cp center], zmFactor)];
+          [path lineToPoint: [cp center]];
+          [displayPath lineToPoint: GRpointZoom([cp center], zmFactor)];
         }
       else
         {
-          [myPath curveToPoint: GRpointZoom([cp center], zmFactor)
+          [path curveToPoint: [cp center]
+                 controlPoint1: handle1.firstHandle
+                 controlPoint2: handle2.secondHandle];
+          [displayPath curveToPoint: GRpointZoom([cp center], zmFactor)
                  controlPoint1: GRpointZoom(handle1.firstHandle, zmFactor)
                  controlPoint2: GRpointZoom(handle2.secondHandle, zmFactor)];
           [cp setPointPosition:GRPointMiddle];
@@ -682,7 +634,7 @@ static double k = 0.025;
 
 - (void)moveAddingCoordsOfPoint:(NSPoint)p
 {
-    int i;
+  NSUInteger i;
 
     for(i = 0; i < [controlPoints count]; i++)
     {
@@ -694,7 +646,7 @@ static double k = 0.025;
 
 - (void)setZoomFactor:(CGFloat)f
 {
-    int i;
+  NSUInteger i;
 
     zmFactor = f;
     for(i = 0; i < [controlPoints count]; i++)
@@ -736,7 +688,7 @@ static double k = 0.025;
     return YES;
 
   /* mypath represents the Path in the current zoom, so it needs to be converted */
-  if([myPath containsPoint: GRpointZoom(p, zmFactor)])
+  if([displayPath containsPoint: GRpointZoom(p, zmFactor)])
     return YES;
   
   return NO;
@@ -764,7 +716,7 @@ static double k = 0.025;
   found = NO;
 
   i = 0;
-  while (i < [controlPoints count] || !found)
+  while (i < [controlPoints count] && !found)
     {
       if([controlPoints objectAtIndex: i] == aPoint)
         found = YES;
@@ -798,28 +750,35 @@ static double k = 0.025;
   NSUInteger i;
   NSBezierPath *bzp;
   CGFloat linew;
-    
+  NSBezierPath *pathToDraw;
+  
   if(![controlPoints count] || !visible)
     return;
 
-  linew =  linewidth * zmFactor;
-
+  linew =  linewidth;
+  pathToDraw = path;
+  if ([[NSGraphicsContext currentContext] isDrawingToScreen])
+    {
+      linew = linewidth * zmFactor;
+      pathToDraw = displayPath;
+    }
+    
   bzp = [NSBezierPath bezierPath];
   if(filled)
     {
       [NSGraphicsContext saveGraphicsState];
       [fillColor set];
-      [myPath fill];
+      [pathToDraw fill];
       [NSGraphicsContext restoreGraphicsState];
     }
   if(stroked)
     {
       [NSGraphicsContext saveGraphicsState];
-      [myPath setLineJoinStyle:linejoin];
-      [myPath setLineCapStyle:linecap];
-      [myPath setLineWidth:linew];
+      [pathToDraw setLineJoinStyle:linejoin];
+      [pathToDraw setLineCapStyle:linecap];
+      [pathToDraw setLineWidth:linew];
       [strokeColor set];
-      [myPath stroke];
+      [pathToDraw stroke];
       [NSGraphicsContext restoreGraphicsState];
     }
     
