@@ -2,7 +2,7 @@
  Project: Graphos
  GRCircle.m
 
- Copyright (C) 2009-2017 GNUstep Application Project
+ Copyright (C) 2009-2018 GNUstep Application Project
 
  Author: Ing. Riccardo Mottola
 
@@ -76,127 +76,32 @@
             inView:(GRDocView *)aView
         zoomFactor:(CGFloat)zf
 {
-  NSMutableDictionary *props;
-  NSString *str;
-  NSArray *linearr;
-  float strokeCol[4];
-  float fillCol[4];
-  float strokeAlpha;
-  float fillAlpha;
-  NSColor *color;
-  id obj;
-
-  props = [NSMutableDictionary dictionaryWithCapacity:2];
-
-  [props setObject:[description objectForKey: @"posx"]  forKey:@"posx"];  
-  [props setObject:[description objectForKey: @"posy"]  forKey:@"posy"];  
-  [props setObject:[description objectForKey: @"height"]  forKey:@"height"];  
-  [props setObject:[description objectForKey: @"width"]  forKey:@"width"];  
-
-  str = [description objectForKey: @"strokecolor"];
-  linearr = [str componentsSeparatedByString: @" "];
-  strokeAlpha = [[description objectForKey: @"strokealpha"] floatValue];
-  color = nil;
-  if ([linearr count] == 3)
-    {
-      strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
-      strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
-      strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
-      color = [NSColor colorWithCalibratedRed: strokeCol[0]
-					green: strokeCol[1]
-					 blue: strokeCol[2]
-					alpha: strokeAlpha];
-    }
-  else
-    {
-      strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
-      strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
-      strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
-      strokeCol[3] = [[linearr objectAtIndex: 3] floatValue];
-      color = [NSColor colorWithDeviceCyan: strokeCol[0]
-				   magenta: strokeCol[1]
-				    yellow: strokeCol[2]
-				     black: strokeCol[3]
-				     alpha: strokeAlpha];
-      color = [color colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
-    }
-  if (color)
-    [props setObject:color forKey:@"strokecolor"];
-
-  str = [description objectForKey: @"fillcolor"];
-  linearr = [str componentsSeparatedByString: @" "];
-  fillAlpha = [[description objectForKey: @"fillalpha"] floatValue];
-  color = nil;
-  if ([linearr count] == 3)
-    {
-      fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
-      fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
-      fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
-      color = [NSColor colorWithCalibratedRed: fillCol[0]
-					    green: fillCol[1]
-					     blue: fillCol[2]
-					    alpha: fillAlpha];
-    }
-  else
-    {
-      fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
-      fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
-      fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
-      fillCol[3] = [[linearr objectAtIndex: 3] floatValue];
-      color = [NSColor colorWithDeviceCyan: fillCol[0]
-				   magenta: fillCol[1]
-				    yellow: fillCol[2]
-				     black: fillCol[3]
-				     alpha: fillAlpha];
-      color = [color colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
-    }
-  if (color)
-    [props setObject:color forKey:@"fillcolor"];
-
-  /* only add the circle property if we really have it */
-  obj = [description objectForKey: @"circle"];
-  if ([obj isKindOfClass:[NSString class]])
-    {
-      obj = [NSNumber numberWithInt:[obj intValue]];
-      [props setObject:obj forKey:@"circle"];
-    }
-
-  
-  obj = [description objectForKey: @"stroked"];
-  if ([obj isKindOfClass:[NSString class]])
-    obj = [NSNumber numberWithInt:[obj intValue]];
-  [props setObject:obj forKey:@"stroked"];
-
-  obj = [description objectForKey: @"filled"];
-  if ([obj isKindOfClass:[NSString class]])
-    obj = [NSNumber numberWithInt:[obj intValue]];	
-  [props setObject:obj forKey:@"filled"];
-
-  obj = [description objectForKey: @"visible"];
-  if ([obj isKindOfClass:[NSString class]])
-    obj = [NSNumber numberWithInt:[obj intValue]];	
-  [props setObject:obj forKey:@"visibile"];
-  
-  obj = [description objectForKey: @"locked"];
-  if ([obj isKindOfClass:[NSString class]])
-    obj = [NSNumber numberWithInt:[obj intValue]];
-  [props setObject:obj forKey:@"locked"];
-
-  obj = [description objectForKey: @"rotation"];
-  if (obj)
-    [props setObject:obj forKey: @"rotation"];
-
-  [props setObject:[description objectForKey: @"flatness"] forKey: @"flatness"];
-  [props setObject:[description objectForKey: @"linejoin"] forKey: @"linejoin"];
-  [props setObject:[description objectForKey: @"linecap"] forKey: @"linecap"];
-  [props setObject:[description objectForKey: @"miterlimit"] forKey: @"miterlimit"];
-  [props setObject:[description objectForKey: @"linewidth"] forKey: @"linewidth"];
-  
-  self = [self initInView:aView zoomFactor:zf withProperties:props];
+  self = [super initFromData:description inView:aView zoomFactor:zf];
   if(self)
     {
+      id obj;
+
+      pos = NSMakePoint([[description objectForKey: @"posx"]  floatValue],
+			[[description objectForKey: @"posy"]  floatValue]);
+      size = NSMakeSize([[description objectForKey: @"width"]  floatValue],
+			[[description objectForKey: @"height"]  floatValue]);
+      bounds = GRMakeBounds(pos.x, pos.y, size.width, size.height);
+
+      isCircle = NO;
+      obj = [description objectForKey: @"circle"];
+      if (obj)      
+	isCircle = [obj boolValue];
+
+      rotation = 0;
+      obj = [description objectForKey: @"rotation"];
+      if (obj)      
+	rotation = [obj floatValue];
+
+      startControlPoint = [[GRObjectControlPoint alloc] initAtPoint: pos zoomFactor:zf];
+      endControlPoint = [[GRObjectControlPoint alloc] initAtPoint: NSMakePoint(pos.x + size.width, pos.y + size.height) zoomFactor:zf];
+      [self setZoomFactor: zf];
     }
-    return self;
+  return self;
 }
 
 - (id)copyWithZone:(NSZone *)zone
@@ -404,12 +309,19 @@
   CGFloat w,h;
   CGFloat minLength;
   CGFloat linew;
+  NSRect drawBounds;
 
-  linew = linewidth * zmFactor;
+  drawBounds = bounds;
+  linew = linewidth;
+  if ([[NSGraphicsContext currentContext] isDrawingToScreen])
+    {
+      drawBounds = boundsZ;
+      linew = linewidth * zmFactor;
+    }
 
-  center = NSMakePoint(NSMidX(boundsZ), NSMidY(boundsZ));
-  w = NSWidth(boundsZ);
-  h = NSHeight(boundsZ);
+  center = NSMakePoint(NSMidX(drawBounds), NSMidY(drawBounds));
+  w = NSWidth(drawBounds);
+  h = NSHeight(drawBounds);
   if (w > h)
     minLength = h;
   else
@@ -420,7 +332,7 @@
   if (isCircle)
     [bzp appendBezierPathWithArcWithCenter:center radius:radius startAngle:0 endAngle:360];
   else
-    [bzp appendBezierPathWithOvalInRect: boundsZ];
+    [bzp appendBezierPathWithOvalInRect: drawBounds];
     
   if(filled)
     {
