@@ -2,7 +2,7 @@
  Project: Graphos
  GRDocView.m
 
- Copyright (C) 2000-2017 GNUstep Application Project
+ Copyright (C) 2000-2018 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -39,6 +39,9 @@
 #define UNDO_ACTION_OBJPROPS @"Change Object Properties"
 #define UNDO_ACTION_CP_SYMMETRIC @"Change Point to Symmetric"
 #define UNDO_ACTION_CP_CUSP @"Change Point to Cusp"
+#define UNDO_ACTION_CP_EXTRACT @"Change Point by Extracting"
+#define UNDO_ACTION_CP_OVERLAP @"Change Point by Overlapping"
+#define UNDO_ACTION_CP_DELETE @"Delete Point"
 
 #define ZOOM_FACTORS 13
 #define STD_ZOOM_INDEX 5
@@ -429,7 +432,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   [self setFrame: pageRect];
   [self setNeedsDisplay:YES];
 }
-                                                              
+
 - (void)deleteSelectedObjects
 {
     id obj;
@@ -983,6 +986,8 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
     {
       [(GRBezierControlPoint *)[points objectAtIndex: i] setSymmetricalHandles:YES];
     }
+  [path remakePath];
+  [self setNeedsDisplay: YES];
 }
 
 - (void)changePointsOfCurrentPathToCusp:(id)sender
@@ -1024,26 +1029,159 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
     {
       [(GRBezierControlPoint *)[points objectAtIndex: i] setSymmetricalHandles:NO];
     }
+  [path remakePath];
+  [self setNeedsDisplay:YES];
 }
 
+- (void)changePointsOfCurrentPathByOverlap:(id)sender
+{
+  NSUndoManager *uMgr;
+  NSUInteger i;
+  NSArray *points;
+  GRBezierPath *path;
+
+  path = nil;
+  for(i = 0; i < [objects count]; i++)
+    {
+      GRDrawableObject *obj;
+      obj = [objects objectAtIndex: i];
+      
+      if([[obj editor] isSelected] && [obj isKindOfClass:[GRBezierPath class]])
+        {
+          path = (GRBezierPath *)obj;
+        }
+    }
+  
+  if (!path)
+    return;
+
+  points = [(GRBezierPathEditor *)[path editor] selectedControlPoints];
+  if (!points || [points count] == 0)
+    return;
+
+  uMgr = [self undoManager];
+  /* save the method on the undo stack, but stack actions */
+  if ([[uMgr undoActionName] isEqualToString: UNDO_ACTION_CP_OVERLAP] == NO)
+    {
+      [self saveCurrentObjectsDeep];
+      [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+      [uMgr setActionName: UNDO_ACTION_CP_OVERLAP];
+    }
+
+  for (i = 0; i < [points count]; i++)
+    {
+      [(GRBezierControlPoint *)[points objectAtIndex: i] overlapHandles];
+    }
+  [path remakePath];
+  [self setNeedsDisplay: YES];
+}
+
+- (void)changePointsOfCurrentPathByExtract:(id)sender
+{
+  NSUndoManager *uMgr;
+  NSUInteger i;
+  NSArray *points;
+  GRBezierPath *path;
+
+  path = nil;
+  for(i = 0; i < [objects count]; i++)
+    {
+      GRDrawableObject *obj;
+      obj = [objects objectAtIndex: i];
+      
+      if([[obj editor] isSelected] && [obj isKindOfClass:[GRBezierPath class]])
+        {
+          path = (GRBezierPath *)obj;
+        }
+    }
+  
+  if (!path)
+    return;
+
+  points = [(GRBezierPathEditor *)[path editor] selectedControlPoints];
+  if (!points || [points count] == 0)
+    return;
+
+  uMgr = [self undoManager];
+  /* save the method on the undo stack, but stack actions */
+  if ([[uMgr undoActionName] isEqualToString: UNDO_ACTION_CP_EXTRACT] == NO)
+    {
+      [self saveCurrentObjectsDeep];
+      [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+      [uMgr setActionName: UNDO_ACTION_CP_EXTRACT];
+    }
+
+  for (i = 0; i < [points count]; i++)
+    {
+      [(GRBezierControlPoint *)[points objectAtIndex: i] extractHandles];
+    }
+  [path remakePath];
+  [self setNeedsDisplay: YES];
+}
+
+- (IBAction)deletePointsOfCurrentPath:(id)sender
+{
+  NSUndoManager *uMgr;
+  NSUInteger i;
+  NSArray *points;
+  GRBezierPath *path;
+    
+  path = nil;
+  for(i = 0; i < [objects count]; i++)
+    {
+      GRDrawableObject *obj;
+      obj = [objects objectAtIndex: i];
+        
+      if([[obj editor] isSelected] && [obj isKindOfClass:[GRBezierPath class]])
+	{
+	  path = (GRBezierPath *)obj;
+	}
+    }
+    
+  if (!path)
+    return;
+    
+  points = [(GRBezierPathEditor *)[path editor] selectedControlPoints];
+  if (!points || [points count] == 0)
+    return;
+    
+  uMgr = [self undoManager];
+  /* save the method on the undo stack, but stack actions */
+  if ([[uMgr undoActionName] isEqualToString: UNDO_ACTION_CP_DELETE] == NO)
+    {
+      [self saveCurrentObjectsDeep];
+      [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+      [uMgr setActionName: UNDO_ACTION_CP_DELETE];
+    }
+    
+  for (i = 0; i < [points count]; i++)
+    {
+      [path deletePoint: [points objectAtIndex: i]];
+    }
+  [path remakePath];
+  [self setNeedsDisplay: YES];
+}
+
+
 - (void)subdividePathAtPoint:(NSPoint)p splitIt:(BOOL)split
 {
   id obj;
   NSUInteger i;
 
-    for(i = 0; i < [objects count]; i++)
+  
+  for(i = 0; i < [objects count]; i++)
     {
-        obj = [objects objectAtIndex: i];
-        if([obj isKindOfClass: [GRBezierPathEditor class]])
+      obj = [objects objectAtIndex: i];
+      if([obj isKindOfClass: [GRBezierPath class]] && [[obj editor] isSelected])
         {
-            if([obj onPathBorder: p])
+	  if([obj onPathBorder: p])
             {
-                [obj selectForEditing];
-                [obj subdividePathAtPoint: p splitIt: split];
-                break;
+	      [obj subdividePathAtPoint: p splitIt: split];
+	      break;
             }
         }
     }
+  [self setNeedsDisplay:YES];
 }
 
 - (NSDictionary *)selectionProperties
@@ -1909,6 +2047,103 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   return [super performKeyEquivalent:theEvent];
 }
 
+- (BOOL)validateMenuItem:(NSMenuItem *)mi
+{
+  NSUInteger i;
+  NSUInteger selectedPaths;
+  NSUInteger selectedObjs;
+  SEL action;
+  
+  selectedPaths = 0;
+  selectedObjs = 0;
+
+  if ([[[mi menu] title] isEqualToString:@"Zoom"])
+    return YES;
+  
+  for(i = 0; i < [objects count]; i++)
+    {
+      GRDrawableObject *obj;
+      obj = [objects objectAtIndex: i];
+    
+      if([[obj editor] isSelected])
+        {
+          selectedObjs++;
+          if ([obj isKindOfClass:[GRBezierPath class]])
+            selectedPaths++;
+        }
+    }
+
+  action = [mi action];
+  if (sel_isEqual(action, @selector(paste:)))
+    return YES;
+      
+  if (sel_isEqual(action, @selector(copy:)) || sel_isEqual(action, @selector(cut:)) || sel_isEqual(action, @selector(delete:)))
+    {
+      if (selectedObjs)
+        return YES;
+      else
+        return NO;
+    }
+
+  if (sel_isEqual(action, @selector(changePointsOfCurrentPathToSymmetric:)))
+    {
+      if ([[NSApp delegate] currentToolType] == whitearrowtool && (selectedPaths == 1))
+        {
+          return YES;
+        }
+      else
+        return NO;
+    }
+  
+  if (sel_isEqual(action, @selector(changePointsOfCurrentPathToCusp:)))
+    {
+      if ([[NSApp delegate] currentToolType] == whitearrowtool && (selectedPaths == 1))
+        {
+          return YES;
+        }
+      else
+        return NO;
+    }
+  
+  if (sel_isEqual(action, @selector(changePointsOfCurrentPathByOverlap:)))
+    {
+      if ([[NSApp delegate] currentToolType] == whitearrowtool && (selectedPaths == 1))
+        {
+          return YES;
+        }
+      else
+        return NO;
+    }
+  
+  if (sel_isEqual(action, @selector(changePointsOfCurrentPathByExtract:)))
+    {
+      if ([[NSApp delegate] currentToolType] == whitearrowtool && (selectedPaths == 1))
+        {
+          return YES;
+        }
+      else
+        return NO;
+    }
+
+  if (sel_isEqual(action, @selector(deletePointsOfCurrentPath:)))
+    {
+      if ([[NSApp delegate] currentToolType] == whitearrowtool && (selectedPaths == 1))
+        {
+          return YES;
+        }
+      else
+        return NO;
+    }
+  if (sel_isEqual(action, @selector(moveSelectedObjectsToFront:)) || sel_isEqual(action, @selector(moveSelectedObjectsToBack:)))
+    {
+      if (selectedObjs)
+        return YES;
+      else
+        return NO;
+    }
+  return NO;
+}
+
 /* override the menu for special context menus
 
    While editing a path, display the handle options
@@ -1919,10 +2154,8 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
     {
       NSUInteger i;
       NSUInteger selectedPaths;
-      GRBezierPath *path;
       
       selectedPaths = 0;
-      path = nil;
       for(i = 0; i < [objects count]; i++)
         {
           GRDrawableObject *obj;
@@ -1931,7 +2164,6 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
           if([[obj editor] isSelected] && [obj isKindOfClass:[GRBezierPath class]])
             {
               selectedPaths++;
-              path = (GRBezierPath *)obj;
             }
         }
 
@@ -1942,7 +2174,14 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
           
           
           menu = [[NSMenu alloc] initWithTitle: NSLocalizedString(@"Handles", @"")];
-          
+
+	  menuItem = [[NSMenuItem alloc] init];
+          [menuItem setTitle: NSLocalizedString(@"Delete", @"")];
+          [menuItem setTarget: self];      
+          [menuItem setAction: @selector(deletePointsOfCurrentPath:)];     
+          [menu addItem: menuItem];
+          [menuItem release];
+	  
           menuItem = [[NSMenuItem alloc] init];
           [menuItem setTitle: NSLocalizedString(@"Symmetric", @"")];
           [menuItem setTarget: self];      
@@ -1956,6 +2195,20 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
           [menuItem setTarget: self];      
           [menuItem setAction: @selector(changePointsOfCurrentPathToCusp:)];     
           [menuItem release];
+
+          menuItem = [[NSMenuItem alloc] init];
+          [menuItem setTitle: NSLocalizedString(@"Overlap to Segment", @"")];
+          [menu addItem: menuItem];
+          [menuItem setTarget: self];      
+          [menuItem setAction: @selector(changePointsOfCurrentPathByOverlap:)];     
+          [menuItem release];
+
+          menuItem = [[NSMenuItem alloc] init];
+          [menuItem setTitle: NSLocalizedString(@"Extract to Round", @"")];
+          [menu addItem: menuItem];
+          [menuItem setTarget: self];      
+          [menuItem setAction: @selector(changePointsOfCurrentPathByExtract:)];     
+          [menuItem release];
           
           [menu autorelease];
           return menu;
