@@ -2,7 +2,7 @@
  Project: Graphos
  GRDrawableObject.h
 
- Copyright (C) 2008-2013 GNUstep Application Project
+ Copyright (C) 2008-2015 GNUstep Application Project
 
  Author: Ing. Riccardo Mottola
 
@@ -31,6 +31,10 @@
 #import <Foundation/Foundation.h>
 #import <AppKit/NSColor.h>
 
+#ifndef MAC_OS_X_VERSION_10_4
+#define MAC_OS_X_VERSION_10_4 1040
+#endif
+
 #if !defined (GNUSTEP) &&  (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4) && !defined(CGFloat)
 #define NSUInteger unsigned
 #define NSInteger int
@@ -51,6 +55,9 @@
   NSColor *strokeColor;
 }
 
+/** return a fresh allocated editor corresponding to the current object */
+- (GRObjectEditor *)allocEditor;
+
 - (id)initInView:(GRDocView *)aView zoomFactor:(CGFloat)zf withProperties:(NSDictionary *)properties;
 - (id)initFromData:(NSDictionary *)description inView:(GRDocView *)aView zoomFactor:(CGFloat)zf;
 
