@@ -25,6 +25,10 @@
 
 #import <Foundation/Foundation.h>
 
+#ifndef MAC_OS_X_VERSION_10_4
+#define MAC_OS_X_VERSION_10_4 1040
+#endif
+
 #if !defined (GNUSTEP) &&  (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4) && !defined(CGFloat)
 #define NSUInteger unsigned
 #define NSInteger int
@@ -34,7 +38,7 @@
 @class GRBoxEditor;
 
 
-@interface GRObjectControlPoint : NSObject
+@interface GRObjectControlPoint : NSObject <NSCopying>
 {
   BOOL isActiveHandle;
   BOOL isSelect;
