@@ -32,11 +32,8 @@
 @interface GRText : GRDrawableObject
 {
     NSString *str;
-    NSFont *font;
+    NSDictionary *parAttributes;
     NSPoint pos;
-    float fsize;
-    NSTextAlignment align;
-    float parspace;
     NSSize size;
     NSRect bounds;
     float scalex, scaley;
@@ -55,8 +52,6 @@
         zoomFactor:(CGFloat)zf;
 
 
-- (NSString *)fontName;
-
 - (void)setString:(NSString *)aString attributes:(NSDictionary *)attrs;
 
 - (void)edit;
