@@ -2,7 +2,7 @@
  Project: Graphos
  GRTextEditorView.m
 
- Copyright (C) 2000-2013 GNUstep Application Project
+ Copyright (C) 2000-2017 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -32,14 +32,13 @@
          withString:(NSString *)string
          attributes:(NSDictionary *)attributes
 {
-  NSString *firstStr;
-  NSFont *f;
-  NSParagraphStyle *pstyle;
-  
-
   self = [super initWithFrame: frameRect];
   if(self)
     {
+      NSString *firstStr;
+      NSFont *f;
+      NSParagraphStyle *pstyle;
+    
       controlsView = [[NSView alloc] initWithFrame: NSMakeRect(0, 270, 500, 40)];
       [controlsView setAutoresizingMask: ~NSViewMaxYMargin & ~NSViewHeightSizable];
 
@@ -77,8 +76,11 @@
         [rightButt setAction: @selector(changeTextAlignment:)];
         [controlsView addSubview: rightButt];
 
-        cancelButt = [[NSButton alloc] initWithFrame: NSMakeRect(360, 5, 60, 30)];
+        cancelButt = [[NSButton alloc] initWithFrame: NSMakeRect(350, 5, 80, 30)];
         [cancelButt setButtonType: NSMomentaryLight];
+#if defined(__APPLE__)
+        [cancelButt setBezelStyle:NSRoundedBezelStyle];
+#endif
         [cancelButt setTitle: @"Cancel"];
         [cancelButt setTarget: self];
         [cancelButt setAction: @selector(okCancelPressed:)];
@@ -86,6 +88,9 @@
 
         okButt = [[NSButton alloc] initWithFrame: NSMakeRect(430, 5, 60, 30)];
         [okButt setButtonType: NSMomentaryLight];
+#if defined(__APPLE__)
+        [okButt setBezelStyle:NSRoundedBezelStyle];
+#endif
         [okButt setTitle: @"Ok"];
         [okButt setTarget: self];
         [okButt setAction: @selector(okCancelPressed:)];
@@ -115,9 +120,9 @@
             [leftButt setState: NSOnState];
             fontSize = 12;
 	    font = [NSFont systemFontOfSize:fontSize];
-            parSpace = fontSize * 1.2;
+            pstyle = [NSMutableParagraphStyle defaultParagraphStyle];
+            parSpace = [pstyle paragraphSpacing];
         }
-
         [self addSubview: controlsView];
 
         scrollView = [[NSScrollView alloc] initWithFrame:NSMakeRect(0, 0, 500, 260)];
@@ -214,7 +219,6 @@
       [self updateFontPreview:fontField :newFont];
       font = newFont;
       [font pointSize];
-      parSpace = fontSize * 1.2;
       [theText setFont: font];
       [theText setNeedsDisplay: YES];
     }
